# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from typing import Iterator

from troubadix.helper.patterns import (
    SpecialScriptTag,
    get_special_script_tag_pattern,
)
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckScriptCallsMandatory(FileContentPlugin):
    name = "check_script_calls_mandatory"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """
        This script checks for the existence of the following mandatory script
        calls:
        - script_name
        - script_version
        - script_category
        - script_family
        - script_copyright
        """
        if nasl_file.suffix == ".inc":
            return

        mandatory_calls = [
            SpecialScriptTag.NAME,
            SpecialScriptTag.VERSION,
            SpecialScriptTag.CATEGORY,
            SpecialScriptTag.FAMILY,
            SpecialScriptTag.COPYRIGHT,
        ]

        for call in mandatory_calls:
            if not get_special_script_tag_pattern(call).search(file_content):
                yield LinterError(
                    "VT does not contain the following mandatory call: "
                    f"'script_{call.value}'",
                    file=nasl_file,
                    plugin=self.name,
                )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.helper import CURRENT_ENCODING
from troubadix.plugin import LinterFix
from troubadix.plugins.last_modification import CheckLastModification

from . import PluginTestCase


class TestUpdateModificationDate(PluginTestCase):
    def test_fix_last_modifaction_date(self):
        with self.create_directory() as testdir:
            nasl_file = testdir / "test.nasl"
            content = (
                'script_tag(name:"last_modification", '
                'value:"2021-03-24 10:08:26 +0000 (Wed, 24 Mar 2021)");\n'
                'script_version("2021-03-24T10:08:26+0000");\n'
            )
            nasl_file.write_text(content, encoding=CURRENT_ENCODING)
            fake_context = self.create_file_plugin_context(
                nasl_file=nasl_file, file_content=content
            )
            plugin = CheckLastModification(fake_context)

            results = list(plugin.run())

            self.assertEqual(len(results), 0)

            results = list(plugin.fix())

            self.assertEqual(len(results), 1)
            self.assertIsInstance(results[0], LinterFix)

            new_content = nasl_file.read_text(encoding=CURRENT_ENCODING)
            self.assertNotEqual(content, new_content)

    def test_ignore(self):
        nasl_file = Path(__file__).parent / "fail.nasl"

        content = nasl_file.read_text(encoding=CURRENT_ENCODING)
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckLastModification(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 0)

        results = list(plugin.fix())
        self.assertEqual(len(results), 0)

        new_content = nasl_file.read_text(encoding=CURRENT_ENCODING)
        self.assertEqual(content, new_content)

    def test_ignore_missing_script_version(self):
        nasl_file = Path(__file__).parent / "fail2.nasl"

        content = nasl_file.read_text(encoding=CURRENT_ENCODING)

        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckLastModification(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 0)

        results = list(plugin.fix())
        self.assertEqual(len(results), 0)

        new_content = nasl_file.read_text(encoding=CURRENT_ENCODING)
        self.assertEqual(content, new_content)



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" updating the modification time in VTs that have been touched/edited """

import datetime
import re
from typing import Iterator

from troubadix.helper import CURRENT_ENCODING
from troubadix.plugin import FilePlugin, LinterFix, LinterResult


class CheckLastModification(FilePlugin):
    name = "check_last_modification"

    def run(self) -> Iterator[LinterResult]:
        # this plugin is deprecated and will be removed in future

        if False:  # pylint: disable=using-constant-test
            # ensure this is a generator method
            yield

        if self.context.nasl_file.suffix == ".inc":
            return

        self.new_file_content = None

        # update modification date
        file_content = self.context.file_content

        tag_template = 'script_tag(name:"last_modification", value:"{date}");'
        mod_pattern = (
            r"script_tag\(name:\"last_modification\", value:\"(.*)\"\);"
        )

        match = re.search(
            pattern=mod_pattern,
            string=file_content,
        )
        if not match:
            return

        old_datetime = match.groups()[0]

        now = datetime.datetime.now(datetime.timezone.utc)
        # get that date formatted correctly:
        # "2021-03-24 10:08:26 +0000 (Wed, 24 Mar 2021)"
        correctly_formatted_datetime = (
            f"{now:%Y-%m-%d %H:%M:%S %z (%a, %d %b %Y)}"
        )

        file_content = file_content.replace(
            tag_template.format(date=old_datetime),
            tag_template.format(date=correctly_formatted_datetime),
        )

        # update script version
        version_template = 'script_version("{date}");'
        vers_template = r"script_version\(\"(.*)\"\);"

        match = re.search(
            pattern=vers_template,
            string=file_content,
        )
        if not match:
            return

        old_version = match.groups()[0]
        # get that date formatted correctly:
        # "2021-03-24T10:08:26+0000"
        correctly_formatted_version = f"{now:%Y-%m-%dT%H:%M:%S%z}"

        self.new_file_content = file_content.replace(
            version_template.format(date=old_version),
            version_template.format(date=correctly_formatted_version),
        )
        self.old_version = old_version
        self.new_version = correctly_formatted_version
        self.old_datetime = old_datetime
        self.new_datetime = correctly_formatted_datetime

    def fix(self) -> Iterator[LinterResult]:
        if self.new_file_content:
            self.context.nasl_file.write_text(
                self.new_file_content, encoding=CURRENT_ENCODING
            )

            yield LinterFix(
                f"Replaced modification_date {self.old_datetime} "
                f"with {self.new_datetime} and script_version "
                f"{self.old_version} with {self.new_version}.",
                file=self.context.nasl_file,
                plugin=self.name,
            )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from typing import Iterator, Union

from troubadix.helper import CURRENT_ENCODING
from troubadix.helper.patterns import get_common_tag_patterns
from troubadix.plugin import FilePlugin, LinterFix, LinterResult, LinterWarning

# import magic


# ;                 can not be displayed in GSA, within
#                   (summary|impact|affected|insight|vuldetect|solution)
# | =               are delimiter in the internal VT cache
#                   (e.g. Tag1=Foo|Tag2=Bar|Tag3=Baz)
#                   in all script_tag(name:"", value:"")
FORBIDDEN_CHARS = ["|", "=", ";"]

# replacement character will be a whitespace
REPLACE_CHAR = " "


def check_match(match: re.Match) -> Union[str, None]:
    """Replace all FORBIDDEN characters with the replace character defined.
    Returns:
        The fixed string or None"""
    if match and match.group("value") is not None:
        changes: bool = False
        line: str = match.group(0)
        value: str = match.group("value")
        for char in FORBIDDEN_CHARS:
            if char in value:
                changes = True
                value = value.replace(char, REPLACE_CHAR)
        if changes:
            return line.replace(match.group("value"), value)
    return None


class CheckIllegalCharacters(FilePlugin):
    name = "check_illegal_characters"

    def run(self) -> Iterator[LinterResult]:
        """
        Currently the following chars are not allowed in
        every script_tag(name:"", value:"") :
        """

        if self.context.nasl_file.suffix == ".inc":
            return

        pattern = get_common_tag_patterns()
        file_content = self.context.file_content

        self.new_file_content = None

        tag_matches = pattern.finditer(file_content)
        if tag_matches:
            for match in tag_matches:
                if match and match.group(0) is not None:
                    new_tag = check_match(match)
                    if new_tag:
                        file_content = file_content.replace(
                            match.group(0), new_tag
                        )
                        self.new_file_content = file_content
                        yield LinterWarning(
                            f"Found illegal character in {match.group(0)}",
                            file=self.context.nasl_file,
                            plugin=self.name,
                        )

    def fix(self) -> Iterator[LinterResult]:
        if not self.new_file_content:
            return

        self.context.nasl_file.write_text(
            self.new_file_content, encoding=CURRENT_ENCODING
        )

        yield LinterFix(
            "Replaced Illegal Characters.",
            file=self.context.nasl_file,
            plugin=self.name,
        )
